unit Packages;

interface

uses
  SysUtils, Token, myRobot;

type
  TPackages = class (TObject)
  public
    constructor Create(Data: String);
    destructor Destroy(); override;
    procedure buildPackageList(Data: String);

    procedure setDistance(loc: Point; dis: Integer);

    function getNextDest(): Point;
    function getBestPackageList(capacity: Integer; var pDest: Point): String;
  private
    id: array of Integer;               // Package ID's
    destX: array of Integer;            // Package Dest X
    destY: array of Integer;            // Package Dest Y
    weight: array of Integer;           // Package Weight
    dist: array of Integer;              // Package Distance from Location
  published
end;

implementation

constructor TPackages.Create(Data: String);
begin
  buildPackageList(data);
end;

destructor TPackages.Destroy();
begin
  setLength(id, 0);
  setLength(destX, 0);
  setLength(destY, 0);
  setLength(weight, 0);
  setLength(dist, 0);
end;

procedure TPackages.buildPackageList(Data: String);
var
  i: Integer;
  myToken: TToken;
  count: Integer;
begin
  myToken := TToken.Create(Data, ' ', '+');
  count := myToken.countTokens();
  if (count > 0) and ((count mod 4) = 0) then begin
    setLength(id, count div 4);
    setLength(destX, count div 4);
    setLength(destY, count div 4);
    setLength(weight, count div 4);
    setLength(dist, count div 4);
    i := 0;
    while (myToken.hasMoreTokens()) do begin
      id[i] := StrToInt(myToken.nextToken());
      destX[i] := StrToInt(myToken.nextToken()) - 1;
      destY[i] := StrToInt(myToken.nextToken()) - 1;
      weight[i] := StrToInt(myToken.nextToken());
      dist[i] := -2;
      inc(i);
    end;
  end
  else if ((count mod 4) > 0) then begin
    WriteLn('Bad Package Data, Integer count incorrect');
  end;
end;

// Sets all packages to the given delivery point
procedure TPackages.setDistance(loc: Point; dis: Integer);
var i: Integer;
begin
  for i := 0 to high(id) do begin
    if (destX[i] = loc.x) and
       (destY[i] = loc.y) then begin
      dist[i] := dis;
    end;
  end;
end;

// Function returns our next dilivery point
function TPackages.getNextDest(): Point;
var i: Integer;
begin
  Result.x := -1;
  Result.y := -1;
  for i := 0 to high(id) do begin
    if (id[i] > 0) and (dist[i] = -2) then begin
      Result.x := destX[i];
      Result.y := destY[i];
    end;
  end;
end;

function TPackages.getBestPackageList(capacity: Integer; var pDest: Point): String;
var
  i: Integer;
  lowDist: Integer;
  addWeight: Integer;
  lowWeight: Integer;
  lowWeightLoc: Integer;
  added: Boolean;
begin
  lowDist := -1;
  Result := '';

  // find our shortest delivery route;
  for i := 0 to high(id) do begin
    if (lowDist = -1) or (dist[i] < lowDist) then begin
      lowDist := dist[i];
      pDest.x := destX[i];
      pDest.y := destY[i];
    end;
  end;

  // Now find the lowest weight packages we can pick up. In this game
  // smaller is better, if you get pushed the package you drop won't
  // be of as much effect if you get the larger packages
  addWeight := 0;
  added := true;
  while (added) do begin
    added := false;
    lowWeight := -1;
    lowWeightLoc := -1;
    for i := 0 to high(id) do begin
      if (dist[i] = lowDist) and (id[i] > 0) and
         ((lowWeight = -1) or (weight[i] < lowWeight)) and
         (weight[i]+addWeight <= capacity) then begin
        lowWeight := weight[i];
        lowWeightLoc := i;
        added := true;
      end;
    end;
    if (added) then begin
      inc(addWeight, weight[lowWeightLoc]);
      if (Result = '') then
        Result := IntToStr(id[lowWeightLoc])
      else
        Result := Result + ' ' + IntToStr(id[lowWeightLoc]);
      id[lowWeightLoc] := -1;
    end;
  end;
end;

end.
